/*****************************************************************************************************************************
--
--    File Name    : cam_data_ddr_write.v

--    Description  : This module generates DDR arbiter control inputs
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module camdata_ddr_write #(
                           /******************************************************************************************************************************/
                           /***********************************************Parameters*********************************************************************/
                           /******************************************************************************************************************************/
                            parameter X_Y_RES_BIT_WIDTH = 11,
                            parameter g_AXI_AWIDTH = 32)
                          (
                          /******************************************************************************************************************************/
                          /***********************************************Module IO**********************************************************************/
                          /******************************************************************************************************************************/ 
                           input wire clock,
                           input wire nreset,
                           input wire start_wr_proc,
                           input wire frame_done,
                           input wire[(g_AXI_AWIDTH - 1) : 0] image_base_addr,
                           output reg wr_req,
                           output reg[(g_AXI_AWIDTH - 1) : 0] ddr_address,
                           output reg[15 : 0] bytes_to_write,
                           input wire wr_ack,
                           input wire wr_done,
                           input wire[(X_Y_RES_BIT_WIDTH - 1) : 0] obj_w
                           );
  parameter[3 : 0] idle = 4'd0,
                   sample_obj_hw = 4'd1,
                   write_trans_calc = 4'd2,
                   ddr_wr_req_gen = 4'd3,
                   wait_arb_ack = 4'd4,
                   wait_arb_done = 4'd5;
  //Registers declarations  
  reg[3 : 0] write_ctrl_fsm;
  reg[(g_AXI_AWIDTH - 1) : 0] s_image_base_addr;
  reg[(X_Y_RES_BIT_WIDTH - 1) : 0] line_cntr;
  reg[(X_Y_RES_BIT_WIDTH - 1) : 0] obj_w_int;
  reg s_r_frame_done;
  reg s_r_frame_done_d;
  reg s_r_frame_done_2d;
  wire s_r_frame_done_dtc;

  /******************************************************************************************************************************/
  /**************************Process to Register Input data*********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge (clock) or
      negedge (nreset))
    begin
      if (!nreset)
        begin
          s_r_frame_done <= 1'b0;
          s_r_frame_done_d <= 1'b0;
          s_r_frame_done_2d <= 1'b0;
        end
      else
        begin
          s_r_frame_done <= frame_done;
          s_r_frame_done_d <= s_r_frame_done;
          s_r_frame_done_2d <= s_r_frame_done_d;
        end
    end
  assign s_r_frame_done_dtc = ~s_r_frame_done_2d & s_r_frame_done_d;
  /******************************************************************************************************************************/
  /**************************FSM to Generate Write Request***********************************************************************/
  /******************************************************************************************************************************/
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          write_ctrl_fsm <= idle;
        end
      else
        begin
          case (write_ctrl_fsm)
            idle:
              begin
                if (start_wr_proc)
                  write_ctrl_fsm <= sample_obj_hw;
                else
                  write_ctrl_fsm <= idle;
              end
            sample_obj_hw:
              begin
                write_ctrl_fsm <= write_trans_calc;
              end
            write_trans_calc:
              begin
                write_ctrl_fsm <= ddr_wr_req_gen;
              end
            ddr_wr_req_gen:
              begin
                write_ctrl_fsm <= wait_arb_ack;
              end
            wait_arb_ack:
              begin
                if (wr_ack)
                  write_ctrl_fsm <= wait_arb_done;
                else
                  write_ctrl_fsm <= wait_arb_ack;
              end
            wait_arb_done:
              begin
                if (wr_done)
                  write_ctrl_fsm <= idle;
                else
                  write_ctrl_fsm <= wait_arb_done;
              end
            default:
              write_ctrl_fsm <= idle;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          wr_req <= 1'b0;
          ddr_address <= { (g_AXI_AWIDTH + 1) { 1'b0 } };
          bytes_to_write <= 16'd0;
          s_image_base_addr <= { g_AXI_AWIDTH { 1'b0 } };
          line_cntr <= { X_Y_RES_BIT_WIDTH { 1'b0 } };
          obj_w_int <= { 11 { 1'b0 } };
        end
      else
        begin
          case (write_ctrl_fsm)
            idle:
              begin
                if (s_r_frame_done_dtc == 1'b1)
                  begin
                    line_cntr <= { X_Y_RES_BIT_WIDTH { 1'b0 } };
                    s_image_base_addr <= image_base_addr;
                  end
              end
            sample_obj_hw:
              begin
                obj_w_int <= obj_w;
              end
            write_trans_calc:
              begin
                ddr_address <= s_image_base_addr + (line_cntr * obj_w_int * 11'd4);
                bytes_to_write <= obj_w_int * 11'd4;
              end
            ddr_wr_req_gen:
              begin
                wr_req <= 1'b1;
              end
            wait_arb_ack:
              begin
                if (wr_ack)
                  wr_req <= 1'b0;
                else
                  wr_req <= wr_req;
              end
            wait_arb_done:
              begin
                if (wr_done)
                  begin
                    line_cntr <= line_cntr + 1'b1;
                  end
                else
                  begin
                    line_cntr <= line_cntr;
                  end
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
