/*****************************************************************************************************************************
--
--    File Name    : downsampler.v

--    Description  : This module implements downsampler for YCbCr 4:4:4 to 4:2:2--                                  

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module downsampler #(
/******************************************************************************************************************************/
/***********************************************Parameters*********************************************************************/
/******************************************************************************************************************************/
                     parameter g_YCbCr_DATA_BIT_WIDTH  = 8
                   )
                   (
/******************************************************************************************************************************/
/***********************************************Module IO**********************************************************************/
/******************************************************************************************************************************/
                   //System Clock
                   input wire                                  CLOCK_I,
                   //System Reset  
                   input wire                                  RESET_N_I,
                   //Luminance data input
                   input wire [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] Y_in_i,
                   //Chroma Data Inputs
                   input wire [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] Cb_in_i,
                   input wire [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] Cr_in_i,
                   //Data Input enable
                   input wire                                  data_enable_i, 
                   //Horizontal Sync
                   input wire                                  horz_sync_i,
                   //Vertical Sync
                   input wire                                  vert_sync_i,
                   //Horizontal data active signal
                   input wire                                  hactive_i,
                   //Vertical data active signal
                   input wire                                  vactive_i,
                   //Luminance data output
                   output reg [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] Y_out_o,
                   //Chroma Data Output
                   output reg [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] C_out_o,
                   //Data Input enable
                   output reg                                  data_enable_o,
                   //Horizontal Sync
                   output reg                                  horz_sync_o,
                   //Vertical Sync
                   output reg                                  vert_sync_o,
                   //Horizontal data active signal
                   output reg                                  hactive_o,
                   //Vertical data active signal
                   output reg                                  vactive_o                   
                   );
//Registers declaration
  reg [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] Y_reg;
  reg [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] U_reg;
  reg [(g_YCbCr_DATA_BIT_WIDTH-1) : 0] V_reg;
  reg h_in_d1;
  reg v_in_d1;
  reg odd_pixel;
  reg hsync_d1;
  reg vsync_d1;
  reg dvi_de_d1;

/******************************************************************************************************************************/
/********************************Process to register all the inputs************************************************************/
/******************************************************************************************************************************/
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin: LATCH_INPUTS
      if (!RESET_N_I)
        begin
          Y_reg     <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          U_reg     <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          V_reg     <= { g_YCbCr_DATA_BIT_WIDTH { 1'b0 } };
          h_in_d1   <= 1'b0;
          v_in_d1   <= 1'b0;
          hsync_d1  <= 1'b0;
          vsync_d1  <= 1'b0;
          dvi_de_d1 <= 1'b0;
        end
      else
        begin
          Y_reg     <= Y_in_i;
          U_reg     <= Cb_in_i;
          V_reg     <= Cr_in_i;
          h_in_d1   <= hactive_i;
          v_in_d1   <= vactive_i;
          hsync_d1  <= horz_sync_i;
          vsync_d1  <= vert_sync_i;
          dvi_de_d1 <= data_enable_i;
        end
    end
/******************************************************************************************************************************/
/********************************Process to downsample the inputs to generate the output***************************************/
/******************************************************************************************************************************/
  always
    @(posedge CLOCK_I or
      negedge RESET_N_I)
    begin: DOWNSAMPLE
      if (!RESET_N_I)
        begin
          Y_out_o       <= 8'h10;
          C_out_o       <= 8'h80;
          odd_pixel     <= 1'b0;
          hactive_o     <= 1'b0;
          vactive_o     <= 1'b0;
          horz_sync_o   <= 1'b0;
          vert_sync_o   <= 1'b0;
          data_enable_o <= 1'b0;
        end
      else
        if (h_in_d1)
          begin
            Y_out_o       <= Y_reg;
            C_out_o       <= odd_pixel ? V_reg : U_reg;
            odd_pixel     <= ~odd_pixel;
            hactive_o     <= h_in_d1;
            vactive_o     <= v_in_d1;
            horz_sync_o   <= hsync_d1;
            vert_sync_o   <= vsync_d1;
            data_enable_o <= dvi_de_d1;
          end
        else
          begin
            Y_out_o       <= 8'h10;
            C_out_o       <= 8'h80;
            odd_pixel     <= 1'b0;
            hactive_o     <= h_in_d1;
            vactive_o     <= v_in_d1;
            horz_sync_o   <= hsync_d1;
            vert_sync_o   <= vsync_d1;
            data_enable_o <= dvi_de_d1;
          end
    end
endmodule
